#ifndef DOMAIN_LIBRARY_H
#define DOMAIN_LIBRARY_H

#include "address_space.h"
#include "bus.h"
#include "component.h"
#include "connection.h"

#include <string>
#include <vector>


namespace domain {

    using namespace std;

    class Library {
        std::string name;
        std::string libraryInfo;
        std::string header;
        std::string componentDirectory;
        std::optional<std::string> componentHeader;

        std::vector<AddressSpace> addressSpaces;
        std::vector<Component> components;
        std::vector<Bus> buses;
        std::vector<Connection> connections;

        std::map<std::string, std::string> messages;

    public:
        Library(string name, string libraryInfo, string header, string componentDirectory,
                std::optional<string> componentHeader,
                vector<AddressSpace> addressSpaces, vector<Component> components, vector<Bus> buses,
                vector<Connection> connection, map<string, string> messages);

        std::string getName();

        std::string getLibraryInfo();

        std::string getHeader();

        std::string getComponentDirectory();

        std::optional<std::string> getComponentHeader();

        std::vector<AddressSpace> getAddressSpaces();

        std::vector<Component> getComponents();

        std::vector<Bus> getBuses();

        std::vector<Connection> getConnections();

        bool hasComponent(std::string name);

        bool hasBus(std::string name);

        std::map<std::string, std::string> getMessages();

        AddressSpace &getAddressSpace(std::string name);

        Component &getComponent(std::string name);

        Bus &getBus(std::string bus);

        std::string getMessage(std::string key);

        bool hasConnection(ConnectionComponent component, std::string bus);

        std::optional<Connection> getConnection(ConnectionComponent component, std::string bus);

        bool hasConnection(ConnectionComponent component, std::string bus, ConnectionComponent secondComponent);

        std::optional<Connection> getConnection(ConnectionComponent component, std::string bus, ConnectionComponent secondComponent);

        bool hasConnection(ConnectionComponent component, ConnectionComponent secondComponent);

        std::optional<Connection> getConnection(ConnectionComponent component, ConnectionComponent secondComponent);

    };

} // namespace domain

#endif // DOMAIN_LIBRARY_H
